// Copyright 2019  github.com. All rights reserved.
// Use of this source code is governed by  github.com.
package model

import (
	"smart-alarm/consts"
	"smart-alarm/errors"
	"time"

	"github.com/jaryhe/gopkgs/logger"
	jsoniter "github.com/json-iterator/go"
	"go.uber.org/zap"

	"github.com/jinzhu/gorm"
)

// 替换encoding/json包
var json = jsoniter.ConfigCompatibleWithStandardLibrary

type TAlarmRule struct {
	Id             int64  `gorm:"primary_key"`
	ProjectId      int64  `gorm:"column:project_id"`
	Sn             string `gorm:"sn"`
	ContinuePeriod int32  `gorm:"column:continue_period"`
	AlarmCount     int32  `gorm:"column:alarm_count"`
	SilencePeriod  int32  `gorm:"column:silence_period"`
	IsOn           bool   `gorm:"column:is_on"`
	CreatedAt      string `gorm:"column:created_at"`
	UpdatedAt      string `gorm:"column:updated_at"`
}

func (TAlarmRule) TableName() string {
	return "t_alarm_rule"
}

func (p *TAlarmRule) Insert(db *gorm.DB) error {
	timeNow := time.Now().Format(consts.TimeSecondLayOut)
	p.CreatedAt = timeNow
	p.UpdatedAt = timeNow
	err := db.Create(p).Error
	if err != nil {
		fields, _ := json.MarshalToString(*p)
		logger.Error("mysql",
			zap.String("sql", "insert into t_alarm_rule"),
			zap.String("fields", fields),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *TAlarmRule) Delete(db *gorm.DB, filter map[string]interface{}) error {
	err := db.Where(filter).Delete(p).Error
	if err != nil {
		fields, _ := json.MarshalToString(filter)
		logger.Error("mysql",
			zap.String("sql", "delete from t_alarm_rule"),
			zap.String("fields", fields),
			zap.String("error", err.Error()))
	}
	return err
}

// 通过结构体变量更新字段值, gorm库会忽略零值字段。就是字段值等于0, nil, "", false这些值会被忽略掉，不会更新。如果想更新零值，可以使用map类型替代结构体。
func (p *TAlarmRule) UpdateSome(db *gorm.DB, filed map[string]interface{}) error {
	if filed == nil {
		return errors.ParamsError
	}
	timeNow := time.Now().Format(consts.TimeSecondLayOut)
	filed["updated_at"] = timeNow
	err := db.Model(p).Updates(filed).Error
	if err != nil {
		fields, _ := json.MarshalToString(filed)
		logger.Error("mysql",
			zap.String("sql", "update t_alarm_rule"),
			zap.String("fields", fields),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *TAlarmRule) Query(db *gorm.DB, filter map[string]interface{}) error {
	err := db.Where(filter).Find(p).Error
	if err != nil {
		fields, _ := json.MarshalToString(filter)
		logger.Error("mysql",
			zap.String("sql", "select from t_alarm_rule"),
			zap.String("fields", fields),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *TAlarmRule) QueryAll(db *gorm.DB, filter map[string]interface{}) (list []TAlarmRule, err error) {
	err = db.Where(filter).Find(&list).Error
	if err != nil {
		fields, _ := json.MarshalToString(filter)
		logger.Error("mysql",
			zap.String("sql", "select from t_alarm_rule"),
			zap.String("fields", fields),
			zap.String("error", err.Error()))
	}
	return list, err
}
